Ext.onReady(function(){
	
	//URLs published by our RESTful web service, 
	//see com.myeclipseide.examples.rest.CustomersResource for more details	
	var customerListURL = "jaxrs/Customer/";
	var removeCustomerURL = "jaxrs/Customer/remove/";
	var addCustomerURL = "jaxrs/Customer/add/";
	
    //create the Data Store
    var store = new Ext.data.Store({
        url: customerListURL,	//makes an HTTP GET request to services/customers/ to get a list of customers in XML
        headers: {'Content-Type' : 'application/xml'},
        reader: new Ext.data.XmlReader({
               record: 'customer',
               id: 'customernumber'
           }, [               
               'customernumber',
               'addressline1',
               'city',
               'country',
               'postalcode',
               'creditlimit',               
               'customername',
               'contactfirstname',
               'contactlastname',
               'phone'
           ])
    });

    //create the customer list UI
    var selectionModel = new Ext.grid.CheckboxSelectionModel({singleSelect: true});    
    var grid = new Ext.grid.GridPanel({
        store: store,
        columns: [
            selectionModel,
            {header: "Customer", width: 120, dataIndex: 'customername', sortable: true},
            {header: "Contact", width: 120, dataIndex: 'contactfirstname', sortable: true},
            {header: "Phone", width: 100, dataIndex: 'phone', sortable: true}
        ],
		sm: selectionModel,
		viewConfig: {
			forceFit: true
		},
		 tbar:[{
	            text:'Add Customer',
	            tooltip:'Add a customer to the database',
	            tooltipType: 'title',
	            handler: function() {
			 		customerForm.getForm().reset();
			 		customerWin.show();
		 		}
	        }, '-',{
	            text:'Remove Customer',
	            tooltip:'Remove a customer from the database via an HTTP DELETE request to services/customers/remove/XXX',
	            tooltipType: 'title',
	            handler: function() {
		        	Ext.Ajax.request({
		        		//makes an HTTP DELETE request to services/customers/remove/XXX where XXX is the customer Id
		        		url: removeCustomerURL + selectionModel.getSelected().data.customernumber,
		        		success: function() {
			        		Ext.MessageBox.alert('Status', 'Customer deleted');
			            	store.reload();
		        		},
		        		method: 'DELETE'
		        	});
	        	}
	        }],

        height:400,
		split: true,
		region: 'north'
    });    
    
   
    //form that accepts customer information
    var customerForm = new Ext.FormPanel({
        labelWidth: 75,
        frame:true,
        bodyStyle:'padding:5px 5px 0',
        width: 350,
        defaults: {width: 230},
        defaultType: 'textfield',
       
        items: [{
                fieldLabel: 'Customer name',
                name: 'customername',
                allowBlank:false
            }, {
                fieldLabel: 'Contact first name',
                name: 'contactfirstname'
            }, {
                fieldLabel: 'Address',
                name: 'addressline1'
            }, {
                fieldLabel: 'City',
                name: 'city'
            }, {
                fieldLabel: 'Country',
                name: 'country'
            }, {
            	fieldLabel: 'Phone',
            	name: 'phone'
            }
        ],

        buttons: [{
            text: 'Save',
            handler: function() {
        		Ext.Ajax.request({
        			//makes an HTTP POST request to services/customers/add/
        			//customer data collected in our form is sent as the message body in JSON
	        		url: addCustomerURL,
	        		headers: {'Content-Type' : 'application/json'},
	        		params: Ext.util.JSON.encode(customerForm.getForm().getValues()),
	        		success: function() {
		        		Ext.MessageBox.alert('Status', 'Customer added');
		            	store.reload();
	        		},
	        		failure: function (data) {
	        			Ext.MessageBox.alert('Status', data.responseText);
	        		},
	        		method: 'POST'
	        	});
            	this.findParentByType(Ext.Window).hide();
            },
            tooltip: 'HTTP POST request to jaxrs/customer/add/',
            tooltipType: 'title'
        },{
            text: 'Cancel',
            handler: function() {
            	this.findParentByType(Ext.Window).hide();
            }
        }]
    });

    var customerWin = new Ext.Window({
        applyTo     : 'addcustomer',
        layout      : 'fit',
        height		: 280,
        width		: 350,
        closeAction :'hide',
        plain       : true,
        items       : customerForm
    });
   
	
	//customer detail area
	var customerTplMarkup = [
		'{customername}<br/>',
		'{addressline1}<br/>{city}<br/>{country}<br/>{postalcode}<br/>',
		'Credit limit: {creditlimit}<br/>',
		'Contact: {contactfirstname} {contactlastname}<br/>',
		'Phone: {phone}<br/>'
	];
	var customerTpl = new Ext.Template(customerTplMarkup);

	var ct = new Ext.Panel({
		renderTo: 'customerlist',
		frame: true,
		title: 'Customer List',
		width: 540,
		height: 590,
		layout: 'border',
		items: [
			grid,
			{
				id: 'detailPanel',
				region: 'center',
				title: 'Customer Details',
				bodyStyle: {
					background: '#eee',
					padding: '7px'
				},
				html: 'Please select a customer to see additional details.'
			}
		]
	});
	
	
	grid.getSelectionModel().on('rowselect', function(sm, rowIdx, r) {
		var detailPanel = Ext.getCmp('detailPanel');
		customerTpl.overwrite(detailPanel.body, r.data);		
	});
    store.load();
});